package Sipwise::NGCP::Config::CLI;

use 5.010;
use strict;
use warnings;
use Term::ReadLine;
use Term::ReadLine::Perl;
use Data::Dumper;

# TODO: "use" them dynamically!
use Sipwise::NGCP::Config::CLI::Main;
use Sipwise::NGCP::Config::CLI::Main::Exit;
use Sipwise::NGCP::Config::CLI::Main::Version;
use Sipwise::NGCP::Config::CLI::Main::Configure;
use Sipwise::NGCP::Config::CLI::Main::Configure::Config;
use Sipwise::NGCP::Config::CLI::Main::Configure::Constants;
use Sipwise::NGCP::Config::CLI::Main::Configure::Show;


#use Sipwise::NGCP::Config::CLI::Main::Configure::Get;
#use Sipwise::NGCP::Config::CLI::Main::Configure::Get::Config;
#use Sipwise::NGCP::Config::CLI::Main::Configure::Get::Constants;
#use Sipwise::NGCP::Config::CLI::Main::Configure::Set;
#use Sipwise::NGCP::Config::CLI::Main::Configure::Set::Config;
#use Sipwise::NGCP::Config::CLI::Main::Configure::Set::Constants;
#use Sipwise::NGCP::Config::CLI::Main::Configure::Show;
#use Sipwise::NGCP::Config::CLI::Main::Configure::Show::Config;
#use Sipwise::NGCP::Config::CLI::Main::Configure::Show::Constants;

sub new {
	my $class = shift;
	my $self = {};

	$self->{cmd} = new Sipwise::NGCP::Config::CLI::Main();

	my $term = Term::ReadLine::Perl->new('NGCP CLI');

	$term->ornaments(0);
	if (my $attr = $term->Attribs) {
		$attr->{scroll_nextline} = 1;
		$attr->{basic_word_break_characters}     = " \t\n";
		$attr->{completer_word_break_characters} = " \t\n";
		$attr->{completion_function} = sub {
			my ($text, $line, $start) = @_;
			$text =~ s/^\s+//;
			$line =~ s/^\s+//;

			# construct module name and instantiate
			my @parts = split /\s+/, $line;
			my $module = join '::', map {ucfirst} @parts;
			$module = '::'.$module if length($module);
			$module = $self->{cmd}->get_name().$module;
			# print "parsed module='$module', line='$line', text='$text'\n";
			my $newhandler = undef;
			my $fallback = 0;
			my $loopcount = 0;
			do {
				# instantiate and fetch commands
				#print "inst module='$module', line='$line', text='$text'\n";
				eval { $newhandler = $module->new($text); };
				if(defined $newhandler) {
					my $cmds = $newhandler->get_commands();
					my @auto = ();
					if(
					  (!$newhandler->show_subcommands() && ref $self->{cmd} ne $module) || 
					  ($fallback && !length $text && $newhandler->is_last()) || 
					  keys %{$cmds} == 0) {
						@auto = ();
					}
					else {
						@auto = grep /^$text/i, keys %{$cmds};
					}
					# don't add whitespace after completion
					# if it's a yaml sub-value
					if(@auto == 1 && $auto[0] =~ /\.$/) {
						$readline::rl_completer_terminator_character = '';
					}
					return @auto;
				}
				$module =~ s/::[^:\s]+$//;
				$fallback = 1;
				++$loopcount;
			} while($loopcount < 1000 && ($module ne $self->{cmd}->get_name() || $self->{cmd}->is_last() )); # make sure we don't loop infinitely here, just in case

			# if we're down to the default, use that one instead
			my $cmds = $self->{cmd}->get_commands();
			return grep /^$text/i, keys %{$cmds};
		};
	}
	$self->{term} = $term;
	bless($self, $class);
	return $self;
}

sub print_banner {
print <<END

ngcpsh - The Sipwise NGCP Console

Usage:
<tab><tab> for available commands in each section.
<tab> for autocompletion.
<arrow-up> and <arrow-down> for command history.
<Ctrl+D> or "exit" to exit current command section.

END
}

sub run {
	my $self = shift;
	my $argv = join ' ', @_;

	# do a one-shot if we got args passed via cmdline
#	if(length($argv) > 0) {
#		#my @parts = map {ucfirst} split(/\s+/, $in);
#		my @parts = split /\s+/, $argv;
#		if(@parts) {
#			$self->{cmd}->run(@parts);
#			exit(0);
#		}
#	}

	# else enter loop to accept user input
	$self->print_banner();

	# TODO: doesn't work properly:
	#$SIG{'INT'} = sub {
	#	readline::kill_text(0, 65535);
	#};
	my $base = 'Sipwise::NGCP::Config::CLI::Main';
	while (1) {
		my $in = $self->{term}->readline($self->{cmd}->get_prompt());

		#print "got '$in' from line\n";
		if(defined $in) {
			$in =~ s/^\s+//;
			$in =~ s/\s+$//;
		}

		# happens with ctrl+d, get one level out
		if(!defined $in || ($in eq "exit" && $self->{cmd}->get_name() ne $base)) {
			my $name = $self->{cmd}->get_name();
			next if $name eq $base;
			$name =~ s/::[^:]+$//;
			my $prevcmd = eval { $name->new(); };
			if(defined $prevcmd) {
				$self->{cmd} = $prevcmd;
			} else {
				print "Failed to exit section\n";
			}
			next;
		}

		#$in =~ s/\s+/ /g; # separate by one whitespace
		#$in =~ s/\s*$//g; # strip whitespaces at the end
		#$in =~ s/\s*\<cr\>$//; # strip explicit '<cr>'

		#my @parts = map {ucfirst} split(/\s+/, $in);
		my @parts = split /\s+/, $in;
		next unless(@parts);

		$self->{cmd} = $self->{cmd}->run(@parts);
	}
}

1;
