package Sipwise::NGCP::Config::CLI::Main;

use 5.010;
use strict;
use warnings;

use DBI;

sub new {
	my $class = shift;
	my $self = {};

	$self->{commands} = {
		'exit'      => 'unused help text',
		'configure' => 'unused help text',
		'version'   => 'unused help text',
	};

	$self->{show_subcommands} = 1;
	bless($self, $class);
	return $self;
}


sub get_commands {
	my $self = shift;
	return $self->{commands};
}

sub is_last {
	my $self = shift;
	return $self->{last};
}

sub show_subcommands {
	my $self = shift;
	return $self->{show_subcommands};
}

sub get_name {
	my $self = shift;
	return ref $self;
}

sub get_prompt {
	my $self = shift;
	my $class = ref $self;
	$class =~ s/^Sipwise::NGCP::Config::CLI::Main(::)?//;
	$class =~ s/::/\//g;
	$class = '(' . lc($class) . ')' if(length $class);
	my $prompt = $class.'$ ';
	return $prompt;
}

sub get_parent {
	my $self = shift;
	my $modname = $self->get_name();
	$modname =~ s/::[^:]+$//;
	my $cmd = eval { $modname->new(); };
	unless(defined $cmd) {
		print "Failed to instantiate '$modname'\n";
		return $self;
	}
	return $cmd;
}

sub get_dbh {
	my $self = shift;
	my $dbname = shift;

	my $dbuser = 'sipwise';
	my $dbpass = undef;

	my $fh;
	unless(open $fh, '<', '/etc/mysql/sipwise.cnf') {
		print "Failed to open db config file: $!\n";
		return undef;
	}
	my $conf = join ' ', <$fh>;
	$dbpass = $1 if $conf =~ /^SIPWISE_DB_PASSWORD=\'([^\']+)\'/;
	unless(defined $dbpass) {
		print "Failed to fetch password from db config file\n";
		return undef;
	}
	close $fh;

	my $dbh = DBI->connect('DBI:mysql:'.$dbname.';host=localhost',
		$dbuser, $dbpass, { RaiseError => 0 });
	unless(defined $dbh) {
		print "Failed to connect to db '$dbname': $DBI::errstr\n";
		return undef;
	}
	return $dbh;
}

sub run {
	my $self = shift;
	my @args = @_;

	# by default, just pass instantiation on until no more
	# args are there or until instantiation throws an error

	my $next = shift @args;
	return $self unless defined $next;
	$next = ucfirst $next;

	my $modname = $self->get_name().'::'.$next;
	my $cmd = eval { $modname->new(); };
	unless(defined $cmd) {
	  print "Invalid command '".lc($next)."'\n";
	  return $self;
	}

	# drop any arguments if subcommands are hidden
	unless($cmd->show_subcommands()) {
		@args = ();
	}
	return $cmd->run(@args);
}

1;
