package Sipwise::NGCP::Config::CLI::Main::Configure;

use 5.010;
use strict;
use warnings;
use vars qw(@ISA);
use Sipwise::NGCP::Config::CLI::Main;
@ISA = ("Sipwise::NGCP::Config::CLI::Main");

use Data::Dumper;
use YAML::Tiny;

sub new {
	my $class = shift;
	my $self = {};

	$self->{commands} = {
		'constants'	=> 'unused help text',
		'config'	=> 'unused help text',
		'show'		=> 'unused help text',
	};

	$self->{last} = 0;
	$self->{show_subcommands} = 0;
	bless($self, $class);
	return $self;
}

sub get_prompt {
	my $self = shift;
	my $class = ref $self;
	$class =~ s/^Sipwise::NGCP::Config::CLI::Main::Configure(::)?//;
	$class =~ s/::/\//g;
	$class = '(' . lc($class) . ')' if(length $class);
	my $prompt = $class.'# ';
	return $prompt;

}

sub int_add_commands {
	my $self = shift;
	my $yaml = shift;
	my $current = shift;
	my $seps = shift;

	my $currentseps = $current =~ tr/\.//;

	#print "current='$current'\n";
	if(ref $yaml eq 'HASH' && $currentseps <= $seps) {
		foreach my $k(keys %{$yaml}) {
			$self->int_add_commands($yaml->{$k}, $current.".".$k, $seps);
		}
	} elsif(ref $yaml eq 'ARRAY' && $currentseps <= $seps) {
		my $i = 0;
		foreach my $a(@{$yaml}) {
			$self->int_add_commands($yaml->[0], $current.".".$i, $seps);
			++$i;
		}
	} else {
		$current =~ s/^\.//;
		if(length $current > 0) {
			#print "add '$current'\n";
			$current .= '.' if(ref $yaml eq 'HASH' || ref $yaml eq 'ARRAY');
			$self->{commands}->{$current} = 'dummy';
		}
	}
}

sub get {
	my $self = shift;
	my $arg = join ' ', @_;
	return $self unless(defined $arg);

	$arg =~ s/^\s*//;
	$arg =~ s/\s*$//;
	return $self unless(length $arg);

	# re-read config in each run, in case someone has
	# changed it in the meanwhle
	$self->{yaml} = YAML::Tiny->read($self->{config});
	unless($self->{yaml}) {
		print "Failed to open config file: $YAML::Tiny::errstr\n";
		return $self;
	}


	$arg =~ s/\s+.*$//g; # strip off any additional arguments
	my $yaml = $self->{yaml}->[0];
	my @keys = split /\./, $arg;
	my $array_index = undef;
	foreach my $k(@keys) {
		next unless length($k);
		# in case of an array with an index (foo.0):
		if(ref $yaml eq 'ARRAY' && $k =~ /^\d+$/) {
			if($k >= @{$yaml}) {
				print "Unknown element '$arg'\n";
				return $self;
			}
			$yaml = $yaml->[$k];
		} else {
			unless(exists $yaml->{$k}) {
				print "Unknown element '$arg'\n";
				return $self;
			}
			$yaml = $yaml->{$k};
		}
	}

	my $t = ref $yaml;
	if($t eq 'SCALAR' || $t eq '') {
		print $yaml, "\n";
	} elsif($t eq 'HASH' || $t eq 'ARRAY') {
		my $out = Dumper $yaml;
		$out =~ s/^(\$VAR1 = )/" " x length($1)/eg;
		print $out;
	} else {
		print "Unsupported type '$t'\n";
	}

	return $self;
}

sub set {
	my $self = shift;
	my $arg = shift;
	my $val = shift;
	return $self unless(defined $arg && defined $val);

	$val =~ s/^\s+//;
	$val =~ s/^[\'\"]*(.+)[\'\"]$/$1/;
	return $self unless(length $val);

	# re-read config in each run, in case someone has
	# changed it in the meanwhle
	$self->{yaml} = YAML::Tiny->read($self->{config});
	unless($self->{yaml}) {
		print "Failed to open config file: $YAML::Tiny::errstr\n";
		return $self;
	}
	my $yaml = $self->{yaml}->[0];

	my @keys = split /\./, $arg;
	my $last_key = pop @keys;
	foreach my $k(@keys) {
		next unless length($k);
		if(ref $yaml eq 'ARRAY' && $k =~ /^\d+$/) {
			if($k >= @{$yaml}) {
				print "Unknown element '$arg'\n";
				return $self;
			}
			$yaml = $yaml->[$k];
		} else {
			unless(exists $yaml->{$k}) {
				print "Unknown element '$arg'\n";
				return $self;
			}
			$yaml = $yaml->{$k};
		}
	}

	my $t;
	if(ref $yaml eq 'HASH') {
		unless(exists $yaml->{$last_key}) {
			print "Unknown element '$arg'\n";
			return $self;
		}
		
		$t = ref $yaml->{$last_key};

		if($t eq 'SCALAR' || $t eq '') {
			$yaml->{$last_key} = $val;
		} else {
			print "Setting '$t' not yet supported\n";
			return $self;
		}
	} elsif(ref $yaml eq 'ARRAY' && $last_key =~ /^\d+$/) {
		unless($last_key < @{$yaml}) {
			print "Unknown element '$arg'\n";
			return $self;
		}
		
		$t = ref $yaml->[$last_key];

		if($t eq 'SCALAR' || $t eq '') {
			$yaml->[$last_key] = $val;
		} else {
			print "Setting '$t' not supported\n";
			return $self;
		}
	} else {
		print "Unknown element '$arg'\n";
		return $self;
	}

	print "set '$arg' to '$val'\n";

	$self->{yaml}->write($self->{config});
	if($self->{yaml}->errstr()) {
		print "Failed to write output file: ".$self->{yaml}->errstr()."\n";
	}

	return $self;
}

1;
