package Sipwise::NGCP::Config::CLI::Main::Version;

use 5.010;
use strict;
use warnings;
use vars qw(@ISA);
use Sipwise::NGCP::Config::CLI::Main;
@ISA = ("Sipwise::NGCP::Config::CLI::Main");

use Data::Dumper;

sub new {
	my $class = shift;
	my $self = {};

	$self->{commands} = {};

	bless($self, $class);
	return $self;
}

sub run {
	my $self = shift;
	my $dbh = $self->get_dbh('ngcp');
	return $self->get_parent() unless(defined $dbh);
	my $sth; my $res;

	my $fh = undef;

	# parse version file
	my $versionfile= '/etc/sipwise_ngcp_version';
	open $fh, '<', $versionfile;
	unless($fh) {
		print "Failed to open '$versionfile': $!\n";
		return $self->get_parent();
	}
	my $version = join ' ', <$fh>;
	if($version =~ /System installed\. NGCP version/) {
		my @parts = split /\s+/, $version;
		print "Installed version: $parts[4] installed at $parts[6]\n\n";
	} else {
		print "Invalid string in '$versionfile'\n";
		return $self->get_parent();
	}
	close $fh;

	# parse apt repo file
	my $repofile = '/etc/apt/sources.list.d/sipwise.list';
	$repofile = '/etc/apt/sources.list' unless -f $repofile;
	open $fh, '<', $repofile;
	unless($fh) {
		print "Failed to open '$repofile': $!\n";
		return $self->get_parent();
	}
	my $repoline = 0;
	while(<$fh>) {
		chomp;
		next unless(length $_);
		if(/^\#\s*Sipwise repositor/) {
			$repoline = 1;
			next;
		}
		next unless($repoline);
		s/^deb http:\/\/deb\.sipwise\.com\///;
		if(/^sp(pro|ce)\//) {
			my @parts = split /\s+/, $_;
			print "Repository version: ".$parts[0]."\n\n";
		} elsif(/^autobuild/) {
			my @parts = split /\s+/, $_;
			print "Repository version: ".$parts[1]."\n\n";
		}
	}
	close $fh;


	$sth = $dbh->prepare("select revision from cfg_schema");
	$sth->execute();
	$res = $sth->fetchall_arrayref();
	print "Applied cfg revisions:\n";
	foreach my $row(@{$res}) {
		print "$row->[0] ";
	}
	print "\n\n";
	$sth->finish();

	$sth = $dbh->prepare("select revision from db_schema");
	$sth->execute();
	$res = $sth->fetchall_arrayref();
	print "Applied db revisions:\n";
	foreach my $row(@{$res}) {
		print "$row->[0] ";
	}
	print "\n\n";
	$sth->finish();

	$dbh->disconnect();
	return $self->get_parent();
}

1;
